@@ -31,6 +31,9 @@ class SharedBackendConf(SharedBackendConfUtils):
     def backendClusterMode(self):
         return self._jconf.backendClusterMode()
 
+    def isH2OReplEnabled(self):
+        return self._jconf.isH2OReplEnabled()
+
     def isProgressBarEnabled(self):
         return self._jconf.isProgressBarEnabled()
 
@@ -67,6 +70,9 @@ class SharedBackendConf(SharedBackendConfUtils):
     def ldapLogin(self):
         return self._jconf.ldapLogin()
 
+    def proxyLoginOnly(self):
+        return self._jconf.proxyLoginOnly()
+
     def kerberosLogin(self):
         return self._jconf.kerberosLogin()
 
@@ -199,9 +205,6 @@ class SharedBackendConf(SharedBackendConfUtils):
     def restApiTimeout(self):
         return self._jconf.restApiTimeout()
 
-    def isH2OReplEnabled(self):
-        return self._jconf.isH2OReplEnabled()
-
     def nthreads(self):
         return self._jconf.nthreads()
 
@@ -310,6 +313,14 @@ class SharedBackendConf(SharedBackendConfUtils):
         self._jconf.setLdapLoginDisabled()
         return self
 
+    def setProxyLoginOnlyEnabled(self):
+        self._jconf.setProxyLoginOnlyEnabled()
+        return self
+
+    def setProxyLoginOnlyDisabled(self):
+        self._jconf.setProxyLoginOnlyDisabled()
+        return self
+
     def setKerberosLoginEnabled(self):
         self._jconf.setKerberosLoginEnabled()
         return self
@@ -358,14 +369,14 @@ class SharedBackendConf(SharedBackendConfUtils):
         self._jconf.setLogDir(arg0)
         return self
 
-    def setCloudTimeout(self, arg0):
-        self._jconf.setCloudTimeout(arg0)
-        return self
-
     def setBackendHeartbeatInterval(self, arg0):
         self._jconf.setBackendHeartbeatInterval(arg0)
         return self
 
+    def setCloudTimeout(self, arg0):
+        self._jconf.setCloudTimeout(arg0)
+        return self
+
     def setNodeNetworkMask(self, arg0):
         self._jconf.setNodeNetworkMask(arg0)
         return self
