@@ -17,7 +17,7 @@
 
 from warnings import warn
 from pyspark import keyword_only
-from ai.h2o.sparkling import Initializer
+from ai.h2o.sparkling.Initializer import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
 from ai.h2o.sparkling.ml.algos.H2OSupervisedAlgorithm import H2OSupervisedAlgorithm
 from ai.h2o.sparkling.ml.models.H2ORuleFitMOJOModel import H2ORuleFitMOJOModel
@@ -36,7 +36,6 @@ class H2ORuleFit(H2ORuleFitParams, H2OSupervisedAlgorithm, DistributionBasedH2OT
                  withContributions=False,
                  dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
                  withLeafNodeAssignments=False,
-                 namedMojoOutputColumns=True,
                  convertInvalidNumbersToNa=False,
                  detailedPredictionCol="detailed_prediction",
                  validationDataFrame=None,
@@ -58,6 +57,7 @@ class H2ORuleFit(H2ORuleFitParams, H2OSupervisedAlgorithm, DistributionBasedH2OT
                  distribution="AUTO",
                  labelCol="label",
                  weightCol=None,
+                 maxCategoricalLevels=10,
                  aucType="AUTO"):
         Initializer.load_sparkling_jar()
         super(H2ORuleFit, self).__init__()
