@@ -17,7 +17,7 @@
 
 from warnings import warn
 from pyspark import keyword_only
-from ai.h2o.sparkling import Initializer
+from ai.h2o.sparkling.Initializer import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
 from ai.h2o.sparkling.ml.algos.H2ODRF import H2ODRF
 
@@ -33,7 +33,6 @@ class H2ODRFClassifier(H2ODRF):
                  withContributions=False,
                  dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
                  withLeafNodeAssignments=False,
-                 namedMojoOutputColumns=True,
                  convertInvalidNumbersToNa=False,
                  detailedPredictionCol="detailed_prediction",
                  validationDataFrame=None,
@@ -64,6 +63,7 @@ class H2ODRFClassifier(H2ODRF):
                  minSplitImprovement=1.0E-5,
                  histogramType="AUTO",
                  calibrateModel=False,
+                 calibrationMethod="AUTO",
                  checkConstantResponse=True,
                  modelId=None,
                  nfolds=0,
