@@ -17,7 +17,7 @@
 
 from warnings import warn
 from pyspark import keyword_only
-from ai.h2o.sparkling import Initializer
+from ai.h2o.sparkling.Initializer import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
 from ai.h2o.sparkling.ml.algos.H2OGAM import H2OGAM
 
@@ -32,8 +32,8 @@ class H2OGAMClassifier(H2OGAM):
                  columnsToCategorical=[],
                  keepBinaryModels=False,
                  withContributions=False,
+                 dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
                  withLeafNodeAssignments=False,
-                 namedMojoOutputColumns=True,
                  convertInvalidNumbersToNa=False,
                  detailedPredictionCol="detailed_prediction",
                  validationDataFrame=None,
@@ -42,7 +42,6 @@ class H2OGAMClassifier(H2OGAM):
                  convertUnknownCategoricalLevelsToNa=False,
                  splitRatio=1.0,
                  withStageResults=False,
-                 withDetailedPredictionCol=True,
                  seed=-1,
                  family="AUTO",
                  tweedieVariancePower=0.0,
@@ -76,6 +75,8 @@ class H2OGAMClassifier(H2OGAM):
                  computePValues=False,
                  removeCollinearCols=False,
                  numKnots=None,
+                 splineOrders=None,
+                 splinesNonNegative=None,
                  scale=None,
                  bs=None,
                  keepGamCols=False,
