@@ -17,7 +17,7 @@
 
 from warnings import warn
 from pyspark import keyword_only
-from ai.h2o.sparkling import Initializer
+from ai.h2o.sparkling.Initializer import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
 from ai.h2o.sparkling.ml.algos.H2OAutoML import H2OAutoML
 
@@ -32,8 +32,8 @@ class H2OAutoMLRegressor(H2OAutoML):
                  columnsToCategorical=[],
                  keepBinaryModels=False,
                  withContributions=False,
+                 dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
                  withLeafNodeAssignments=False,
-                 namedMojoOutputColumns=True,
                  convertInvalidNumbersToNa=False,
                  detailedPredictionCol="detailed_prediction",
                  validationDataFrame=None,
@@ -43,9 +43,8 @@ class H2OAutoMLRegressor(H2OAutoML):
                  splitRatio=1.0,
                  monotoneConstraints={},
                  withStageResults=False,
-                 withDetailedPredictionCol=True,
                  projectName=None,
-                 nfolds=5,
+                 nfolds=-1,
                  balanceClasses=False,
                  classSamplingFactors=None,
                  maxAfterBalanceSize=5.0,
@@ -53,6 +52,11 @@ class H2OAutoMLRegressor(H2OAutoML):
                  keepCrossValidationModels=False,
                  keepCrossValidationFoldAssignment=False,
                  exportCheckpointsDir=None,
+                 distribution="AUTO",
+                 tweediePower=1.5,
+                 quantileAlpha=0.5,
+                 huberAlpha=0.9,
+                 customDistributionFunc=None,
                  labelCol="label",
                  foldCol=None,
                  weightCol=None,
