@@ -17,7 +17,7 @@
 
 from warnings import warn
 from pyspark import keyword_only
-from ai.h2o.sparkling import Initializer
+from ai.h2o.sparkling.Initializer import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
 from ai.h2o.sparkling.ml.algos.H2ODeepLearning import H2ODeepLearning
 
@@ -34,7 +34,6 @@ class H2ODeepLearningRegressor(H2ODeepLearning):
                  withContributions=False,
                  dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
                  withLeafNodeAssignments=False,
-                 namedMojoOutputColumns=True,
                  convertInvalidNumbersToNa=False,
                  detailedPredictionCol="detailed_prediction",
                  validationDataFrame=None,
@@ -121,22 +120,14 @@ class H2ODeepLearningRegressor(H2ODeepLearning):
                  stoppingMetric="AUTO",
                  stoppingTolerance=0.0,
                  exportCheckpointsDir=None,
-                 aucType="AUTO",
-                 **kwargs):
+                 aucType="AUTO"):
         Initializer.load_sparkling_jar()
         super(H2ODeepLearning, self).__init__()
         self._java_obj = self._new_java_obj("ai.h2o.sparkling.ml.algos.regression.H2ODeepLearningRegressor", self.uid)
         self._setDefaultValuesFromJava()
         kwargs = Utils.getInputKwargs(self)
         kwargs = self._updateInitKwargs(kwargs)
-        if 'variableImportances' in kwargs:
-            if 'calculateFeatureImportances' not in kwargs:
-                kwargs['calculateFeatureImportances'] = kwargs['variableImportances']
-            del kwargs['variableImportances']
-            warn("The parameter 'variableImportances' is deprecated and will be removed in the version 3.38.")
-        if 'autoencoder' in kwargs:
-            del kwargs['autoencoder']
-            warn("The parameter 'autoencoder' is deprecated and will be removed in the version 3.38.")
+
         if 'interactionPairs' in kwargs:
             warn("Interaction pairs are not supported!")
         self._set(**kwargs)
