@@ -17,7 +17,7 @@
 
 from warnings import warn
 from pyspark import keyword_only
-from ai.h2o.sparkling import Initializer
+from ai.h2o.sparkling.Initializer import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
 from ai.h2o.sparkling.ml.features.H2ODimReductionEstimator import H2ODimReductionEstimator
 from ai.h2o.sparkling.ml.models.H2OPCAMOJOModel import H2OPCAMOJOModel
