@@ -2,14 +2,14 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 import os
-import traceback
 
 import h2o
 from h2o.base import Keyed
-from h2o.exceptions import H2OValueError
+from h2o.display import H2ODisplay, display, format_to_html, format_to_multiline, format_user_tips, print2 as print
+from h2o.exceptions import H2OValueError, H2OTypeError
 from h2o.job import H2OJob
 from h2o.model.extensions import has_extension
-from h2o.plot import decorate_plot_result, get_matplotlib_pyplot, RAISE_ON_FIGURE_ACCESS
+from h2o.plot import decorate_plot_result, get_matplotlib_pyplot, get_matplotlib_cm, get_mplot3d_axes, RAISE_ON_FIGURE_ACCESS
 from h2o.utils.compatibility import *  # NOQA
 from h2o.utils.compatibility import viewitems
 from h2o.utils.metaclass import backwards_compatibility, deprecated_fn, h2o_meta, deprecated_params
@@ -22,8 +22,10 @@ from h2o.utils.typechecks import assert_is_type, assert_satisfies, Enum, is_type
         giniCoef=lambda self, *args, **kwargs: self.gini(*args, **kwargs)
     )
 )
-class ModelBase(h2o_meta(Keyed)):
-    """Base class for all models."""
+class ModelBase(h2o_meta(Keyed, H2ODisplay)):
+    """
+    Base class for all models.
+    """
 
     _options_ = {}    # dict of options declared in implementation
 
@@ -46,7 +48,6 @@ class ModelBase(h2o_meta(Keyed)):
         self._end_time = None
         self._run_time = None
 
-
     @property
     def key(self):
         return self._id
@@ -139,13 +140,6 @@ class ModelBase(h2o_meta(Keyed)):
         """Model training time in milliseconds."""
         return self._run_time
 
-    def __repr__(self):
-        # PUBDEV-2278: using <method>? from IPython caused everything to dump
-        stk = traceback.extract_stack()
-        if not ("IPython" in stk[-2][0] and "info" == stk[-2][2]):
-            self.show()
-        return ""
-
     def predict_leaf_node_assignment(self, test_data, type="Path"):
         """
         Predict on a dataset and return the leaf node assignment (only for tree-based models).
@@ -234,26 +228,73 @@ class ModelBase(h2o_meta(Keyed)):
         >>> # Compute SHAP and pick the top two highest and top two lowest
         >>> m.predict_contributions(fr, top_n=2, bottom_n=2)
         """
-        assert_is_type(output_format, None, Enum("Original", "Compact"))
-        if not isinstance(test_data, h2o.H2OFrame): raise ValueError("test_data must be an instance of H2OFrame")
-        j = H2OJob(h2o.api("POST /4/Predictions/models/%s/frames/%s" % (self.model_id, test_data.frame_id),
-                           data={"predict_contributions": True,
-                                 "predict_contributions_output_format": output_format,
-                                 "top_n": top_n,
-                                 "bottom_n": bottom_n,
-                                 "compare_abs": compare_abs}), "contributions")
-        j.poll()
-        return h2o.get_frame(j.dest_key)
+        if has_extension(self, 'Contributions'):
+            return self._predict_contributions(test_data, output_format, top_n, bottom_n, compare_abs)
+        err_msg = "This model doesn't support calculation of feature contributions."
+        if has_extension(self, 'StandardCoef'):
+            err_msg += " When features are independent, you can use the coef() method to get coefficients"
+            err_msg += " for non-standardized data or coef_norm() to get coefficients for standardized data."
+            err_msg += " You can plot standardized coefficient magnitudes by calling std_coef_plot() on the model."
+        raise H2OTypeError(message=err_msg)
+
+    def row_to_tree_assignment(self, original_training_data):
+        """
+        Output row to tree assignment for the model and provided training data.
+
+        Output is frame of size nrow = nrow(original_training_data) and ncol = number_of_trees_in_model+1 in format:
+         row_id    tree_1    tree_2    tree_3
+              0         0         1         1
+              1         1         1         1
+              2         1         0         0
+              3         1         1         0
+              4         0         1         1
+              5         1         1         1
+              6         1         0         0
+              7         0         1         0
+              8         0         1         1
+              9         1         0         0
+
+        :param H2OFrame original_training_data: Data that was used for model training. Currently there is no validation
+            of the input.
+        :returns: A new H2OFrame made of row to tree assignment output.
+
+        **Note**: Multinomial classification generate tree for each category, each tree use the same sample of the data.
+
+        :examples:
+
+        >>> prostate = "http://s3.amazonaws.com/h2o-public-test-data/smalldata/prostate/prostate.csv"
+        >>> fr = h2o.import_file(prostate)
+        >>> predictors = list(range(2, fr.ncol))
+        >>> m = H2OGradientBoostingEstimator(ntrees=10, seed=1234, sample_rate=0.6)
+        >>> m.train(x=predictors, y=1, training_frame=fr)
+        >>> # Output row to tree assignment
+        >>> m.row_to_tree_assignment(fr)
+        """
+        if has_extension(self, 'RowToTreeAssignment'):
+            return self._row_to_tree_assignment(original_training_data)
+        err_msg = "This model doesn't support calculation of row to tree assignment."
+        raise H2OTypeError(message=err_msg)
 
     def feature_frequencies(self, test_data):
         """
         Retrieve the number of occurrences of each feature for given observations 
-        on their respective paths in a tree ensemble model.
+        on their respective paths in a tree ensemble model. 
         Available for GBM, Random Forest and Isolation Forest models.
 
         :param H2OFrame test_data: Data on which to calculate feature frequencies.
 
         :returns: A new H2OFrame made of feature contributions.
+
+        :examples:
+
+        >>> from h2o.estimators import H2OIsolationForestEstimator
+        >>> h2o_df = h2o.import_file("https://raw.github.com/h2oai/h2o/master/smalldata/logreg/prostate.csv")
+        >>> train,test = h2o_df.split_frame(ratios=[0.75])
+        >>> model = H2OIsolationForestEstimator(sample_rate = 0.1,
+        ...                                     max_depth = 20,
+        ...                                     ntrees = 50)
+        >>> model.train(training_frame=train)
+        >>> model.feature_frequencies(test_data = test)
         """
         if not isinstance(test_data, h2o.H2OFrame): raise ValueError("test_data must be an instance of H2OFrame")
         j = h2o.api("POST /3/Predictions/models/%s/frames/%s" % (self.model_id, test_data.frame_id),
@@ -282,6 +323,35 @@ class ModelBase(h2o_meta(Keyed)):
         j.poll()
         return h2o.get_frame(j.dest_key)
 
+    def calibrate(self, calibration_model):
+        """
+        Calibrate a trained model with a supplied calibration model.
+
+        Only tree-based models can be calibrated.
+
+        :param calibration_model: a GLM model (for Platt Scaling) or Isotonic Regression model trained with the purpose
+            of calibrating output of this model.
+
+        :examples:
+
+        >>> from h2o.estimators.gbm import H2OGradientBoostingEstimator
+        >>> from h2o.estimators.isotonicregression import H2OIsotonicRegressionEstimator
+        >>> df = h2o.import_file("https://s3.amazonaws.com/h2o-public-test-data/smalldata/gbm_test/ecology_model.csv")
+        >>> df["Angaus"] = df["Angaus"].asfactor()
+        >>> train, calib = df.split_frame(ratios=[.8], destination_frames=["eco_train", "eco_calib"], seed=42)
+        >>> model = H2OGradientBoostingEstimator()
+        >>> model.train(x=list(range(2, train.ncol)), y="Angaus", training_frame=train)
+        >>> isotonic_train = calib[["Angaus"]]
+        >>> isotonic_train = isotonic_train.cbind(model.predict(calib)["p1"])
+        >>> h2o_iso_reg = H2OIsotonicRegressionEstimator(out_of_bounds="clip")
+        >>> h2o_iso_reg.train(training_frame=isotonic_train, x="p1", y="Angaus")
+        >>> model.calibrate(h2o_iso_reg)
+        >>> model.predict(train)
+        """
+        if has_extension(self, 'SupervisedTrees'):
+            return self._calibrate(calibration_model)
+        print("Only supervised tree-based models support model calibration")
+
     def is_cross_validated(self):
         """Return True if the model was cross-validated."""
         return self._is_xvalidated
@@ -417,7 +487,7 @@ class ModelBase(h2o_meta(Keyed)):
                 
             If type is ``"auto"`` ("qini"), AUUC is calculated. 
         :param int auuc_nbins: Number of bins for calculation AUUC. Defaults to ``-1``, which means 1000.
-        :returns: An object of class H2OModelMetrics.
+        :returns: An instance of :class:`~h2o.model.metrics_base.MetricsBase` or one of its subclass.
         """
         
         if test_data is None:
@@ -469,6 +539,46 @@ class ModelBase(h2o_meta(Keyed)):
         if "scoring_history" in model and model["scoring_history"] is not None:
             return model["scoring_history"].as_data_frame()
         print("No score history for this model")
+        
+    def negative_log_likelihood(self):
+        """
+        Retrieve model negative likelihood function value from scoring history if exists for GLM model
+        
+        :return: the negative likelihood function value
+        """
+        return self._extract_scoring_history("negative_log_likelihood")
+
+    def average_objective(self):
+        """
+        Retrieve model average objective function value from scoring history if exists for GLM model.  If there is no 
+        regularization, the avearge objective value*obj_reg should equal the neg_log_likelihood value.
+        
+        :return: the average objective function value
+        """
+        return self._extract_scoring_history("objective")
+
+        
+    def _extract_scoring_history(self, value):
+        model = self._model_json["output"]
+        if 'glm' == self.algo:
+            if self.actual_params['generate_scoring_history'] is True:
+                if "scoring_history" in model and model["scoring_history"] is not None:
+                    sc_history = model["scoring_history"]
+                    col_header = sc_history._col_header
+                    if value in col_header:
+                        index_val = col_header.index(value)
+                        scLen = len(sc_history._cell_values)
+                        return sc_history._cell_values[scLen-1][index_val]
+                    else:
+                        print("{0} not available.".format(value))
+                else:
+                    raise H2OValueError("GLM scoring_history is missing.  Cannot extract {0}.".value)
+            else:
+                raise H2OValueError("For now, need to set generate_scoring_history to True to get {0}".format(value))
+        else:
+            raise H2OValueError("{0} is only for GLM models.".format(value))
+
+
 
     def ntrees_actual(self):
         """
@@ -580,55 +690,7 @@ class ModelBase(h2o_meta(Keyed)):
         if "cross_validation_metrics_summary" in model and model["cross_validation_metrics_summary"] is not None:
             return model["cross_validation_metrics_summary"]
         print("No cross-validation metrics summary for this model")
-
-    def summary(self):
-        """Print a detailed summary of the model."""
-        model = self._model_json["output"]
-        if "model_summary" in model and model["model_summary"] is not None:
-            return model["model_summary"]
-        print("No model summary for this model")
-
-    def show_summary(self):
-        summary = self.summary()
-        if summary is not None:
-            print(summary)
-
-    def show(self):
-        """Print innards of model, without regards to type."""
-        if self._future:
-            self._job.poll_once()
-            return
-        if self._model_json is None:
-            print("No model trained yet")
-            return
-        if self.model_id is None:
-            print("This H2OEstimator has been removed.")
-            return
-        model = self._model_json["output"]
-        print("Model Details")
-        print("=============")
-
-        print(self.__class__.__name__, ": ", self._model_json["algo_full_name"])
-        print("Model Key: ", self._id)
-        print()
-
-        self.show_summary()
-
-        # training metrics
-        tm = model["training_metrics"]
-        if tm is not None: tm.show()
-        vm = model["validation_metrics"]
-        if vm is not None: vm.show()
-        xm = model["cross_validation_metrics"]
-        if xm is not None: xm.show()
-        xms = model["cross_validation_metrics_summary"]
-        if xms is not None: xms.show()
-
-        if "scoring_history" in model and model["scoring_history"]:
-            model["scoring_history"].show()
-        if "variable_importances" in model and model["variable_importances"]:
-            model["variable_importances"].show()
-
+        
     def varimp(self, use_pandas=False):
         """
         Pretty print the variable importances, or return them in a list.
@@ -726,6 +788,19 @@ class ModelBase(h2o_meta(Keyed)):
         """Pretty print the coefficents table (includes normalized coefficients)."""
         print(self._model_json["output"]["coefficients_table"])  # will return None if no coefs!
 
+    def get_variable_inflation_factors(self):
+        if self.algo == 'glm':
+            if self.parms['generate_variable_inflation_factors']:
+                tbl = self._model_json["output"]["coefficients_table"]
+                if tbl is None:
+                    return None
+                return {name: vif for name, vif in zip(tbl["names"], tbl["variable_inflation_factor"])}
+            else:
+                raise ValueError("variable inflation factors are generated only when "
+                                 "generate_variable_inflation_factors is enabled.")
+        else:
+            raise ValueError("variable inflation factors are only found in GLM models for numerical predictors.")
+        
     def coef(self):
         """
         Return the coefficients which can be applied to the non-standardized data.
@@ -764,7 +839,7 @@ class ModelBase(h2o_meta(Keyed)):
                                  "compute_p_values=True.")
         else:
             raise ValueError("p-values, z-values and std_error are only found in GLM.")
-            
+        
     def _fillMultinomialDict(self, standardize=False):
         if self.algo == 'gam':
             tbl = self._model_json["output"]["coefficients_table"]
@@ -790,7 +865,7 @@ class ModelBase(h2o_meta(Keyed)):
         """
         Return the R squared for this regression model.
 
-        Will return :math:`R^2` for GLM Models and will return NaN otherwise.
+        Will return :math:`R^2` for GLM Models. 
 
         The :math:`R^2` value is defined to be :math:`1 - MSE / var`, where var is computed as :math:`\sigma * \sigma`.
 
@@ -944,8 +1019,6 @@ class ModelBase(h2o_meta(Keyed)):
         tm = ModelBase._get_metrics(self, train, valid, xval)
         m = {}
         for k, v in viewitems(tm):
-            if not(v == None) and not(is_type(v, h2o.model.metrics_base.H2OBinomialModelMetrics)) and not(is_type(v, h2o.model.metrics_base.H2OMultinomialModelMetrics)):
-                raise H2OValueError("auc() is only available for Binomial and Multinomial classifiers. For Multinomial classifiers is available average AUC value, default is Weighted One-to-Rest AUC.")
             m[k] = None if v is None else v.auc()
         return list(m.values())[0] if len(m) == 1 else m
 
@@ -1004,7 +1077,8 @@ class ModelBase(h2o_meta(Keyed)):
         tm = ModelBase._get_metrics(self, train, valid, xval)
         m = {}
         for k, v in viewitems(tm): 
-            if v is not None and not is_type(v, h2o.model.metrics_base.H2OBinomialModelMetrics) and not is_type(v, h2o.model.metrics_base.H2OMultinomialModelMetrics):
+            if v is not None and not is_type(v, h2o.model.metrics.binomial.H2OBinomialModelMetrics) and not is_type(v,
+                                                                                                                    h2o.model.metrics.multinomial.H2OMultinomialModelMetrics):
                 raise H2OValueError("aucpr() is only available for Binomial and Multinomial classifiers. For Multinomial classifiers is available average PR AUC value, default is Weighted One-to-Rest PR AUC.")
             m[k] = None if v is None else v.aucpr()
         return list(m.values())[0] if len(m) == 1 else m
@@ -1282,14 +1356,14 @@ class ModelBase(h2o_meta(Keyed)):
         to_fig = num_1dpdp + num_2dpdp
         if plot and to_fig > 0:     # plot 1d pdp for now
             plt = get_matplotlib_pyplot(server)
-            cm = _get_matplotlib_cm("Partial dependency plots")
+            cm = get_matplotlib_cm("Partial dependency plots")
             if not plt: 
                 return decorate_plot_result(res=pps, figure=RAISE_ON_FIGURE_ACCESS)
             import matplotlib.gridspec as gridspec
             fig = plt.figure(figsize=figsize)
             gxs = gridspec.GridSpec(to_fig, 1)
             if num_2dpdp > 0: # 2d pdp requested
-                axes_3d = _get_mplot3d_pyplot("2D partial plots")
+                axes_3d = get_mplot3d_axes("2D partial plots")
             fig_plotted = False  # indicated number of figures plotted
             data_index = 0
             target = None
@@ -1840,21 +1914,109 @@ class ModelBase(h2o_meta(Keyed)):
             )
             return varimp
 
-def _get_mplot3d_pyplot(function_name):
-    try:
-        # noinspection PyUnresolvedReferences
-        from mpl_toolkits.mplot3d import Axes3D
-        return Axes3D
-    except ImportError:
-        print("`mpl_toolkits.mplot3d` library is required for function {0}!".format(function_name))
-        return None
+    # --------------------------------
+    # ModelBase representation methods
+    # --------------------------------
+    
+    def _str_items(self, verbosity=None):
+        verbosity = verbosity or 'full'  # default verbosity when printing model
+        # edge cases
+        if self._future:
+            self._job.poll_once()
+            return
+        if self._model_json is None:
+            return "No model available"
+        if self.key is None:
+            return "This model (key={}) has been removed".format(self.key)
 
+        items = []
+        if verbosity in ['full']:
+            items.extend([
+                "Model Details",
+                "============="
+            ])
+        items.extend([
+            "%s : %s" % (self.__class__.__name__, self._model_json["algo_full_name"]),
+            "Model Key: %s" % self.key,
+        ])
+        if verbosity in ['medium', 'full']:
+            summary = self.get_summary()
+            if summary is not None:
+                items.extend(["", summary])
+            
+        if verbosity in ['full']:
+            model = self._model_json["output"]
+            tm = model["training_metrics"]
+            if tm is not None: items.append(tm)
+            vm = model["validation_metrics"]
+            if vm is not None: items.append(vm)
+            xm = model["cross_validation_metrics"]
+            if xm is not None: items.append(xm)
+            xms = model["cross_validation_metrics_summary"]
+            if xms is not None: items.append(xms)
 
-def _get_matplotlib_cm(function_name):
-    try:
-        from matplotlib import cm
-        return cm
-    except ImportError:
-        print('matplotlib library is required for 3D plots for function {0}'.format(function_name))
-        return None
+            if "scoring_history" in model and model["scoring_history"]:
+                items.append(model["scoring_history"])
+            if "variable_importances" in model and model["variable_importances"]:
+                items.append(model["variable_importances"])
+            
+        return items
+    
+    def _str_usage(self, verbosity=None, fmt=None):
+        verbosity = verbosity or 'full'  # default verbosity when printing model
+        if not self._model_json or verbosity == 'short':
+            return ""
+        lines = []
+        if verbosity != 'full':
+            lines.append("Use `model.show()` for more details.")
+        lines.append("Use `model.explain()` to inspect the model.")
+        lines.extend(self._str_usage_custom())
+        return format_user_tips(format_to_multiline(lines), fmt=fmt) if lines else ""
+    
+    def _str_usage_custom(self):
+        """
+        Specific models can override this function to add model-specific user tips
+        :return: a list of strings, each describing a user tip for this model.
+        """
+        return []
+
+    def _str_(self, verbosity=None):
+        items = self._str_items(verbosity)
+        if isinstance(items, list):
+            return format_to_multiline(items)
+        return items
+
+    def _str_pretty_(self, verbosity=None):
+        return self._str_(verbosity)+self._str_usage(verbosity, 'pretty')
+    
+    def _str_html_(self, verbosity=None):
+        items = self._str_items(verbosity)
+        html = format_to_html(items) if isinstance(items, list) else items
+        usage = self._str_usage(verbosity, 'html')
+        return html+usage
+
+    def _summary(self):
+        model = self._model_json["output"]
+        if "model_summary" in model and model["model_summary"] is not None:
+            return model["model_summary"]
+
+    def summary(self):
+        """Deprecated. Please use ``get_summary`` instead"""
+        return self.get_summary()
+    
+    def get_summary(self):
+        """Return a detailed summary of the model."""
+        return self._summary() or "No summary for this model"
+    
+    def show_summary(self):
+        """Print a detailed summary of the model."""
+        summary = self.get_summary()
+        if summary is not None:
+            display(summary)
+            
+    def show(self, verbosity=None, fmt=None):
+        verbosity = verbosity or 'full'  # default verbosity for showing model
+        return display(self, fmt=fmt, verbosity=verbosity)
+
+    # FIXME: find a way to get rid of this awful habit that consists in doing [if data is present return data else print("no data")]
     
